###########################################################################
#                      WonderDesk                                         #
#       "The Perl-based Help Desk solution software"                      #
#                                                                         #
#         File: Mailer.pm                                                 #
#       Author: James Van Horn                                            #
#        Email: James@webwonderland.com                                   #
#          Web: http://www.wonderdesk.com                                 #
#      Version: 2.10                                                      #
#                                                                         #
#  The owner of the script is solely responsible for any problems         #
#  caused by installation of the script or use of the script.             #
#                                                                         #
#  WonderDesk is based off of DBMan, a shareware database engine          #
#  available for download at http://www.gossamer-threads.com/ and is      #
#  used with permission. Your WonderDesk license includes a commercial    #
#  license of DBMan. WonderDesk cannot be be redistributed for any reason.#
###########################################################################

package Mailer;
use strict;
use Socket;
use FileHandle;
use vars qw($VERSION $error $CRLF @ISA @EXPORT);
@ISA    = ();
@EXPORT = qw();
$Mailer::VERSION = '1.0';
$CRLF            = "\015\012";

sub new {
my $this  = shift;
my $class = ref($this) || $this;
my $self  = {};
bless  $self, $class;
return $self->initialize(@_);
}

sub error {
my ($errtype, $e1, $e2) = @_;
my %errors = (
HOSTNOTFOUND     => "SMTP: server '$e1' was not found.",
SOCKFAILED       => "SMTP: socket() failed. reason: $e1",
CONNFAILED       => "SMTP: connect() failed. reason: $e1",
SERVNOTAVAIL     => "SMTP: Service not available.",
COMMERROR        => "SMTP: Unspecified communications error.",
USERUNKOWN       => "SMTP: Local user '$e1' unkown on host '$e2'.",
TRANSFAILED      => "SMTP: Transmission of message failed.",
TOEMPTY          => "No To: field specified.",
NOMSG            => "No message body specified",
SENDMAILNOTFOUND => "Sendmail was not defined or not found: $e1",
NOOPTIONS        => "No options were specified. Be sure to pass a hash ref to send()",
NOTRANSPORT      => "Neither sendmail nor SMTP were specified!",
BADLOG           => "Unable to write to logfile '$e1'. Reason: $e2"
);
$Mailer::error = $errors{$errtype};

return -1;
}

sub initialize {
my $self = shift;

($#_ < 0) and return $self;
if (ref $_[0] eq 'HASH') {
my $key; my $hash = $_[0];
foreach $key (keys %$hash) {
$self->{lc $key} = $hash->{$key};
}
}
else { &error ('NOOPTIONS'); return undef; }

$self->{'from'}  = &clean_email ($self->{'from'});
$self->{'reply'} = &clean_email ($self->{'reply'});
$self->{'to'}    = &clean_email ($self->{'to'});

if (exists $self->{'smtp'} and ($self->{'smtp'} =~ /\w+/)) {
$self->{'mode'}  = 'SMTP';
$self->{'proto'} = (getprotobyname('tcp'))[2];
$self->{'port'}  = getservbyname('smtp', 'tcp');
$self->{'smtp'} =~ s/^\s+//g;
$self->{'smtp'} =~ s/\s+$//g;
$self->{'smtpaddr'} = ($self->{'smtp'} =~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/) ?
pack('C4',$1,$2,$3,$4) : (gethostbyname($self->{'smtp'}))[4];
unless (defined $self->{'smtpaddr'}) {
&error ('HOSTNOTFOUND', $self->{'smtp'});
return undef;
}
}
elsif (exists $self->{'sendmail'} and ($self->{'sendmail'} =~ /\w+/)) {
if ($self->{'sendmail'} =~ /^\S+$/) {
unless (-e $self->{'sendmail'}) {
&error ('SENDMAILNOTFOUND', $self->{'sendmail'});
return undef;
}
$self->{'sendmail'} = $self->{'sendmail'} . " -oeq -t";
}
else {
if ($self->{'sendmail'} =~ /^(.+)\s+.+$/) {
unless (-e $1) {
&error ('SENDMAILNOTFOUND', $1);
return undef;
}
}
}
$self->{'mode'} = 'SENDMAIL';
}
else {
&error ('NOTRANSPORT'); return undef;
}
return $self;
}

sub send {
my $self = shift;

$self->set_options(@_) or return undef;

if ($self->{'mode'} eq 'SMTP') {
return $self->send_smtp();
}
elsif ($self->{'mode'} eq 'SENDMAIL') {
return $self->send_sendmail();
}
else {
&error ('NOTRANSPORT');
return undef;
}
}

sub send_smtp {
my $self = shift;

my $s = &FileHandle::new('FileHandle');
socket($s, AF_INET, SOCK_STREAM, $self->{'proto'})
or (&error('SOCKFAILED', $!) and return undef);
connect($s, pack('Sna4x8', AF_INET, $self->{'port'}, $self->{'smtpaddr'}))
or (&error('CONNFAILED', $!) and return undef);

my($oldfh) = select($s); $| = 1; select($oldfh);

$_ = <$s>; if (/^[45]/) { close $s; &error('SERVNOTAVAIL'); return undef; }

print $s "helo localhost$CRLF";
$_ = <$s>; if (/^[45]/) { close $s; &error('COMMERROR');    return undef; }

print $s "mail from: <", $self->{'from'}, ">$CRLF";
$_ = <$s>; if (/^[45]/) { close $s; &error('COMMERROR');    return undef; }

foreach (split(/,/, $self->{'to'})) {
(/<(.*)>/) ?
print $s "rcpt to: $1$CRLF" :
print $s "rcpt to: <$_>$CRLF";
$_ = <$s>; if (/^[45]/) { close $s; &error('USERUNKNOWN', $self->{'to'}, $self->{'smtp'}); return undef; }
}

print $s "data$CRLF";
$_ = <$s>; if (/^[45]/) { close $s; &error('COMMERROR'); return undef; }

print $s "To: ",       $self->{'to'},      $CRLF;
print $s "From: ",     $self->{'from'},    $CRLF;
print $s "Reply-to: ", $self->{'reply'},   $CRLF        if $self->{'reply'};
print $s $self->{'headers'},               $CRLF        if $self->{'headers'};
print $s "X-Mailer: Mailer::$VERSION (http://www.gossamer-threads.com/scripts/)$CRLF";
print $s "Subject: ",  $self->{'subject'}, $CRLF, $CRLF;

$self->{'msg'}  =~ s/\r//g;
$self->{'msg'}  =~ s/\n/$CRLF/g;

print $s $self->{'msg'};

print $s $CRLF, '.', $CRLF;
$_ = <$s>; if (/^[45]/) { close $s; &error('TRANSFAILED'); return undef; }

print $s "quit", $CRLF;
$_ = <$s>;

close $s;

$self->log_msg() or return undef;

return 1;
}

sub send_sendmail {
my $self = shift;

my $s = &FileHandle::new('FileHandle');
open  ($s, "|$self->{'sendmail'}");
print $s "To: ",       $self->{'to'},      "\n";
print $s "From: ",     $self->{'from'},    "\n";
print $s "Reply-to: ", $self->{'reply'},   "\n"         if $self->{'reply'};
print $s $self->{'headers'},               "\n"     if $self->{'headers'};
print $s "X-Mailer: Mailer::$VERSION (http://www.gossamer-threads.com/scripts/)\n";
print $s "Subject: ",  $self->{'subject'}, "\n\n";
print $s $self->{'msg'};
close $s;

$self->log_msg() or return undef;

return 1;
}

sub set_options {
my $self = shift;
my %changed;
if (ref $_[0] eq 'HASH') {
my $key; my $hash=$_[0];
foreach $key (keys %$hash) {
$self->{lc $key} = $hash->{$key};
$changed{$key}   = 1;
}
}
$self->{'from'}  = &clean_email ($self->{'from'})   if ($changed{'from'});
$self->{'reply'} = &clean_email ($self->{'reply'})  if ($changed{'reply'});
$self->{'to'}    = &clean_email ($self->{'to'})     if ($changed{'to'});
if ($changed{'smtp'} and $self->{'mode'} eq 'SMTP') {
$self->{'smtp'} =~ s/^\s+//g;
$self->{'smtp'} =~ s/\s+$//g;
$self->{'smtpaddr'} = ($self->{'smtp'} =~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/) ?
pack('C4',$1,$2,$3,$4) : (gethostbyname($self->{'smtp'}))[4];
defined($self->{'smtpaddr'}) or &error ('HOSTNOTFOUND', $self->{'smtp'});
}
if ($changed{'sendmail'} and $self->{'mode'} eq 'SENDMAIL') {
if ($self->{'sendmail'} =~ /^\S+$/) {
unless (-e $self->{'sendmail'}) {
&error ('SENDMAILNOTFOUND', $self->{'sendmail'});
return undef;
}
$self->{'sendmail'} = $self->{'sendmail'} . " -oeq -t";
}
else {
if ($self->{'sendmail'} =~ /^(.+)\s+.+$/) {
unless (-e $1) {
&error ('SENDMAILNOTFOUND', $1);
return undef;
}
}
}
}
$self->{'mode'}       or (&error ('NOTRANSPORT') and return undef);
$self->{'to'}         or (&error ('TOEMPTY')     and return undef);
$self->{'msg'}        or (&error ('NOMSG')       and return undef);

return 1;
}

sub clean_email {
my $address = shift;
$address =~ s/^\s+//g;
$address =~ s/\s+$//g;
$address =~ s/,,/,/g;
return $address;
}

sub log_msg {
my $self    = shift;
my $logfile = $self->{'log'};

return 1 unless $logfile;

my $to   = $self->{'to'};
my $from = $self->{'from'};
my $mode = $self->{'mode'};
my $smtp = $self->{'smtp'};
my $send = $self->{'sendmail'};
my $sub  = $self->{'subject'};
my $msg  = $self->{'msg'};
my $time = scalar(localtime);
my $ip   = $ENV{'REMOTE_HOST'};
my $ref  = $ENV{'HTTP_REFERER'};
my $app  = $0;

(-e $logfile) ?
(open (LOG, ">>$logfile") or (&error('BADLOG', $logfile, $!) and return undef)) :
(open (LOG, ">$logfile")  or (&error('BADLOG', $logfile, $!) and return undef));

print LOG qq~
-------------------------------------------
Message sent on $time:

To: $to
From: $from
Subject: $sub

$msg
--------------------
Mode: $mode
SMTP: $smtp
Sendmail: $send
Remote Host: $ip
Remote Ref: $ref
Program: $app

~;
close LOG;
return 1;
}

1;